"""
 Created by Christian Masopust 2011
 Copyright (c) 2011 Christian Masopust. All rights reserved.
"""

import re
import os.path
from operator import itemgetter

from trac.core import *
from trac.db import *
from trac.web.chrome import ITemplateProvider, add_notice, add_warning, add_stylesheet
from trac.admin.web_ui import IAdminPanelProvider
from trac.wiki.model import WikiPage

from testmanager.util import get_page_title


class TestManagerAdmin(Component):
    """
    Provide the functionality to add, edit and create
    templates for TestCases and TestCatalogs
    """

    TYPE_TESTCASE = 'TC'
    TYPE_TESTCATALOG = 'TCAT'

    implements(IAdminPanelProvider, ITemplateProvider)

    # ITemplateProvider methods:
    #
    def get_htdocs_dirs(self):
        from pkg_resources import resource_filename
	return [('testmanager', resource_filename(__name__, 'htdocs'))]

    def get_templates_dirs(self):
        from pkg_resources import resource_filename
	return [resource_filename(__name__, 'templates')]


    # IAdminPanelProvider methods:
    #
    def get_admin_panels(self, req):
        if req.perm.has_permission('TRAC_ADMIN'):
	    yield('testmanager', 'TestManager', 'settings', 'Settings')
	    yield('testmanager', 'TestManager', 'templates', 'Templates')

    def render_admin_panel(self, req, cat, page, component):
        if page == 'settings':
	    return self._render_settings(req, cat, page, component)
	if page == 'templates':
	    return self._render_templates(req, cat, page, component)


    def _render_settings(self, req, cat, page, component):
        req.perm.assert_permission('TRAC_ADMIN')

	data = {}

	if req.method == 'POST':
	    default_days_back = req.args.get('default_days_back')
	    default_interval = req.args.get('default_interval')
	    yui_base_url = req.args.get('yui_base_url')

	    self.env.config.set('testmanager', 'default_days_back', default_days_back)
	    self.env.config.set('testmanager', 'default_interval', default_interval)
	    self.env.config.set('testmanager', 'yui_base_url', yui_base_url)
	    self.env.config.save()
	    add_notice(req, 'Settings saved')

	data['default_days_back'] = self.env.config.get('testmanager', 'default_days_back')
	data['default_interval'] = self.env.config.get('testmanager', 'default_interval')
	data['yui_base_url'] = self.env.config.get('testmanager', 'yui_base_url')

	return 'admin_settings.html', data

    def _render_templates(self, req, cat, page, component):
        req.perm.assert_permission('TRAC_ADMIN')

	for key, value in req.args.items():
	    self.env.log.debug("Key: %s, Value: %s", key, value)

	data = {}

	data['template_overview'] = True
	data['edit_template'] = False

	data['tc_templates'] = self._get_templates(self.TYPE_TESTCASE)
	data['tcat_templates'] = self._get_templates(self.TYPE_TESTCATALOG)
	data['tcat_list'] = self._get_testcatalogs()
	data['tcat_selected'] = self._get_default_testcatalog()

	if req.method == 'POST':
	    
	    # add a TestCase template?
	    if req.args.get('tc_add'):
		tc_name = req.args.get('tc_add_name')
	        self.env.log.debug('Add new TC-template: ' + tc_name)

		if len(tc_name) > 0:
		    if self._template_exists(tc_name, self.TYPE_TESTCASE):
			data['tc_add_name'] = tc_name
		        add_warning(req, 'A TestCase template with that name already exists!')
		    else:
		        data['template_overview'] = False
		        data['edit_template'] = True
		        data['t_edit_type'] = self.TYPE_TESTCASE
		        data['t_edit_name'] = tc_name
		        data['t_edit_action'] = 'ADD'
	        else:
		    add_warning(req, 'Please enter a Template Name first')

	    # add a TestCatalog template?
	    if req.args.get('tcat_add'):
	        tcat_name = req.args.get('tcat_add_name')
		self.env.log.debug('Add new TCat-template: ' + tcat_name)

		if len(tcat_name) > 0:
		    if self._template_exists(tcat_name, self.TYPE_TESTCATALOG):
			data['tcat_add_name'] = tcat_name
		        add_warning(req, 'A TestCatalog template with that name already exists!')
		    else:
		        data['template_overview'] = False
		        data['edit_template'] = True
		        data['t_edit_type'] = self.TYPE_TESTCATALOG
		        data['t_edit_name'] = tcat_name
		        data['t_edit_action'] = 'ADD'
	        else:
		    add_warning(req, 'Please enter a Template Name first')

	    # delete a TestCase template?
	    if req.args.get('tc_del'):
	        tc_sel = req.args.get('tc_sel')
		for t_id in tc_sel:
		    self.env.log.debug("remove testcase template with id: " + t_id)
		    if not self._remove_template(t_id):
			t = self._get_template_by_id(t_id)
		        add_warning(req, 'Error removing ' + t['name'])
		data['tc_templates'] = self._get_templates(self.TYPE_TESTCASE)
		data['tcat_templates'] = self._get_templates(self.TYPE_TESTCATALOG)

	    # delete a TestCatalog template?
	    if req.args.get('tcat_del'):
	        tcat_sel = req.args.get('tcat_sel')
		for t_id in tcat_sel:
		    self.env.log.debug("remove testcatalog template with id: " + t_id)
		    if not self._remove_template(t_id):
			t = self._get_template_by_id(t_id)
		        add_warning(req, 'Error removing ' + t['name'])
		data['tc_templates'] = self._get_templates(self.TYPE_TESTCASE)
		data['tcat_templates'] = self._get_templates(self.TYPE_TESTCATALOG)

	    # save default TestCatalog template
	    if req.args.get('tcat_default_save'):
	        tcat_default = req.args.get('tcat_default')
		if self._save_template_config('TEST_CATALOG_DEFAULT_TEMPLATE', tcat_default):
		    add_notice(req, 'Default TestCatalog template updated')
		    data['tcat_selected'] = tcat_default
		else:
		    add_warning(req, 'Failed to update default TestCatalog template')

	    # save templates for TestCatalogs
	    if req.args.get('tc_templates_save'):
		warning = False
		for key, value in req.args.items():
		    self.env.log.debug("checking key: " + key)
		    if 'TC_TEMPLATE_FOR_TCAT_' in key:
			self.env.log.debug("saving tc-template for: " + key + ", value: " + value)
			if not self._save_template_config(key, value):
			    warning = True
		if warning:
		    add_notice(req, 'Failed to update TestCase templates')
		else:
		    add_notice(req, 'Default TestCase templates updated')
		    data['tcat_list'] = self._get_testcatalogs()

	    # save an edited template?
	    if req.args.get('t_edit_save'):
		t_id = req.args.get('t_edit_id')
	        t_type = req.args.get('t_edit_type')
		t_name = req.args.get('t_edit_name')
		t_desc = req.args.get('t_edit_description')
		t_cont = req.args.get('t_edit_content')
		t_action = req.args.get('t_edit_action')

		self._save_template(t_id, t_name, t_type, t_desc, t_cont, t_action)

		data['template_overview'] = True
		data['edit_template'] = False
		data['tc_templates'] = self._get_templates(self.TYPE_TESTCASE)
		data['tcat_templates'] = self._get_templates(self.TYPE_TESTCATALOG)
		add_notice(req, 'Template saved')

        else:
	    # method 'GET' (template selected for 'edit')
	    if component:
	        t_type = req.args.get('t_type')
		t_id = component
		self.env.log.debug("component: " + component)
		template = self._get_template_by_id(t_id)

		data['t_edit_id'] = template['id']
		data['t_edit_type'] = template['type']
		data['t_edit_name'] = template['name']
		data['t_edit_description'] = template['description']
		data['t_edit_content'] = template['content']
		data['t_edit_action'] = 'EDIT'

		data['template_overview'] = False
		data['edit_template'] = True



	add_stylesheet(req, 'testmanager/css/admin.css')
	return 'admin_templates.html', data


    # get templates from database
    def _get_templates(self, t_type):
	db = self.env.get_db_cnx()
	cursor = db.cursor()
	sql = "SELECT * FROM testmanager_templates WHERE type = '" + t_type + "' ORDER BY name;"
	self.env.log.debug('reading templates: ' + sql)
	cursor.execute(sql)
	items = []
	for row in cursor:
	    temp = { 'id': row[0], 'name': row[1], 'type': row[2], 'description': row[3], 'content': row[4] }
	    items.append(temp)
	return items

    # check if a given template (name) already exists
    def _template_exists(self, name, t_type):
	db = self.env.get_db_cnx()
	cursor = db.cursor()
        sql = "SELECT COUNT(id) FROM testmanager_templates WHERE name = '" + name + "' AND type = '" + t_type + "';"
	cursor.execute(sql)
	row = cursor.fetchone()
	if int(row[0]) > 0:
	    return True
        return False

    # get a single template from database by id
    def _get_template_by_id(self, t_id):
	db = self.env.get_db_cnx()
	cursor = db.cursor()
        sql = "SELECT * FROM testmanager_templates WHERE id = '" + t_id + "';"
	self.env.log.debug("sql: " + sql)
	cursor.execute(sql)
	temp = {}
	for row in cursor:
	    temp = { 'id': row[0], 'name': row[1], 'type': row[2], 'description': row[3], 'content': row[4] }
	    self.env.log.debug(temp)
	return temp

    # get a single template from database by name and type
    def _get_template(self, t_name, t_type):
	db = self.env.get_db_cnx()
	cursor = db.cursor()
        sql = "SELECT * FROM testmanager_templates WHERE name = '" + t_name + "' AND type = '" + t_type + "';"
	cursor.execute(sql)
	temp = {}
	for row in cursor:
	    temp = { 'id': row[0], 'name': row[1], 'type': row[2], 'description': row[3], 'content': row[4] }
	return temp

    # save a template
    def _save_template(self, t_id, t_name, t_type, t_desc, t_cont, t_action):
        db = self.env.get_db_cnx()
	cursor = db.cursor()

	if t_action == 'ADD':
	    t_id = self._get_next_id()
	    self.env.log.debug("next id is: " + t_id)
	    cursor.execute("""
	        INSERT INTO testmanager_templates (id, name, type, description, content) 
		VALUES (%s,%s,%s,%s,%s)
		""", (t_id, t_name, t_type, t_desc, t_cont))
	else:
	    cursor.execute("""
	        UPDATE testmanager_templates 
		SET description = %s, content = %s 
		WHERE id = %s AND name = %s AND type = %s
		""", (t_desc, t_cont, t_id, t_name, t_type))

	db.commit()

    # get next id from testmanager_templates
    def _get_next_id(self):
        db = self.env.get_db_cnx()
	cursor = db.cursor()
	sql = "SELECT id FROM testmanager_templates ORDER BY id DESC LIMIT 1;"
	cursor.execute(sql)
	row = cursor.fetchone()
	return str(int(row[0]) + 1)

    # remove a single template from testmanager_templates
    def _remove_template(self, t_id):
        db = self.env.get_db_cnx()
	cursor = db.cursor()
	try:
	    sql = "DELETE FROM testmanager_templates WHERE id = '%s';" % t_id
	    self.env.log.debug("sql: " + sql)
	    cursor.execute(sql)
	    db.commit()
	except:
	    db.rollback()
	    self.env.log.debug("removing failed")
	    return False
	return True

    # get list of testcatalogs
    def _get_testcatalogs(self):
        db = self.env.get_db_cnx()
	cursor = db.cursor()
	cursor.execute("SELECT * from testcatalog")
	items = []
	for row in cursor:
	    c_id = row[0]
	    c_name = row[1]
	    wikipage = WikiPage(self.env, c_name)
	    c_title = get_page_title(wikipage.text)
	    c_template = self._get_template_from_config(c_id)
	    cat = {'id': c_id, 'name': c_name, 'title': c_title, 'template': c_template}
	    items.append(cat)
        return sorted(items, key=itemgetter('title'))

    # get default testcatlog template from testconfig
    def _get_default_testcatalog(self):
        db = self.env.get_db_cnx()
	try:
	    cursor = db.cursor()
	    cursor.execute("SELECT value FROM testconfig WHERE propname='TEST_CATALOG_DEFAULT_TEMPLATE'")
	    row = cursor.fetchone()
	    return row[0]
	except:
	    db.rollback()
	    return None

    # set default testcatalog template in testconfig
    def _save_template_config(self, propname, tcat_default):
        db = self.env.get_db_cnx()
	try:
	    cursor = db.cursor()
	    sql = "SELECT COUNT(*) FROM testconfig WHERE propname = '" + propname + "'"
	    cursor.execute(sql)
	    row = cursor.fetchone()
	    if int(row[0]) > 0:
	        cursor.execute("""
	                       UPDATE testconfig
			       SET value = %s
			       WHERE propname = %s 
			       """, (tcat_default, propname))
	    else:
	        cursor.execute("""
	                       INSERT INTO testconfig (propname,value)
			       VALUES (%s,%s)
			       """, (propname, tcat_default))
	    db.commit()
	    return True
	except:
	    db.rollback()
	return False

    # get testcatalog template by id
    def _get_template_from_config(self, t_id):
        db = self.env.get_db_cnx()
	cursor = db.cursor()
	sql = "SELECT value FROM testconfig WHERE propname = 'TC_TEMPLATE_FOR_TCAT_" + t_id + "'"
	cursor.execute(sql)
	row = cursor.fetchone()
	if row:
	    return row[0]
	return "0"



